# Workbench Approver

Locks content moderation state transitions until revisions are approved by assigned users.

## Table of contents

- [Requirements](#requirements)
- [Installation](#installation)
- [Configuration](#configuration)
- [Submodules](#submodules)
- [Terminology](#terminology)
- [Using the module](#using-the-module)
- [Components of an approval request](#components-of-an-approval-request)
- [Using the Approve tab](#using-the-approve-tab)
- [Approval statuses](#approval-statuses)
- [Permissions](#permissions)
- [Approvals block](#block)
- [Content moderation changes](#content-moderation-changes)
- [Views integration](#views-integration)
- [Workbench Access integration](#workbench-access-integration)
- [Technical architecture](#technical-architecture)
- [Troubleshooting and FAQ](#troubleshooting-and-faq)
- [Credits](#credits)

## Requirements

- Core `Content moderation` module to handle workflows.
- Contrib [Date filter](https://drupal.org/project/date_filter) module to handle Views date fields.

### Optional modules

- Provides a tab for the [Workbench module](https://drupal.org/project/workbench) editorial workspace.
- Provides enhanced integration with [Workbench Access rules](#workbench-access-integration).

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

Via composer:

```
composer require drupal/workbench_approver
```

Note that `date_filter` is included in this module's `composer.json` file. If it does not auto-install, run `composer require drupal/date_filter`.

## Configuration

The module itself has no configuration, only [permissions](#Permissions).

## Submodules

- **Workbench Approver Access** provides [integration with Workbench Access](#workbench-access-integration) editorial groups.
- **Workbench Approver Lock** locks access to node editing so that only assigned approvers may edit the content.

## Terminology

To fit in with the Workbench suite, the name `Workbench Approver` is used for the module.

- The assignment to approve an entity is called an `approval request`.
- The user assigned to complete an approval request is an `approver`.
- The user who assigned the approval request is an `assigner`.
- The content that ann approval request applies to is a `parent entity`.

## Limitations

- The module only supports the `node` entity type.
  - Extension to other entity types is a desired feature.

## Using the module

Workbench Approver allows sites to require that specific user(s) approve a content revision before it can be transitioned to a new workflow state. Content under moderation can undergo multiple revisions before being approved, and the approval requests will update as each revision is made.

When enabled, the module adds a `Approve` tab to content for any content type that is under moderation. From this tab, user(s) can be assigned to review and approve the latest revision of the content. Until these approval requests are completed, the content can be revised, but may not transition to a new moderation state.

These user(s) are assigned **per entity**, so each piece of content may have a different set of approvers.

Note that approvals are tied to a specific **revision** of the content. If the latest revision is in a moderation state that does not support transitioning to itself, approvals may not be assigned. See [Notes on workflow configuration](#notes-on-workflow-configuration) for more information.

Published revisions are not subject to further approval.

When users create, edit, approve, or request changes with an approval request, they may choose to leave `notes` for others. These notes can help clarify the reason for the approval request, explain the requested changes, or provide other context. These notes are independent of the content revision and are stored with the approval request.

## Components of an approval request

Each approval request is a Drupal entity which is attached to a parent entity. At initial release, only nodes (content) may be used as a parent entity.

An approval request has the following elements, only some of which are exposed to users:

- `Owner` -- the user who created the request.
- `Approver` -- the user responsible for fulfilling the request.
- `Notes` -- any notes about the request or its resolution.
- `Deadline / Due date` -- the date by which the approval request should be resolved. Note that this is a guideline and is not strictly enforced. It is designed to help approvers prioritize their work.
- `Workflow` -- the Content moderation workflow for the parent entity.
- `Moderation state` -- the current moderation state of the parent entity.
- `Content entity type` -- the parent entity type.
- `Content entity id` -- the entity id.
- `Content entity revision id` -- the entity revision id for the approval.
- `Created` -- the date the request was created.
- `Changed` -- the date the request was last updated.
- `Language` -- the language of the parent entity revision.

Note that approval requests are revisioned and translatable.

## Using the Approve tab

The approve tab is divided into four sections, visible to users with the [proper permissions](#permissions):

- `Content summary` -- provided help text, a note about the user's permissions, and information about the latest revision (if the user can view the revision).
- `Active approvals` -- a view of approval requests for the latest revision and a button to add a new request for users with permission to do so.
- `Prior approvals` -- a view of approval requests for the previous revisions of the content.
- `Revision history` -- a view of the content's revision history, if the user has access to view all revisions.

### Active approvals

The active approvals section has a set of `Operations` links, depending on the user's permissions and [the status](#approval-statuses) of the request:

- `Show revision` -- links to the latest revision.
- `View notes` -- opens a panel to show the latest note left on the approval request, if any.
- `Approve revision` -- opens an approval form and its `notes` field.
- `Request changes` -- opens a request changes form and its `notes` field.
- `Edit request` -- allows assigners to change the `deadline`, `required` status, and add notes to an existing request.
- `Delete request` -- allows assigners to remove an approval request from the queue. The request is preserved for auditing purposes.

## Approval statuses

Workbench Approver is designed with auditing in mind, tracking the state of each approval request through the content revision cycle. The following statuses are available:

- **Pending** -- the default status, awaiting a response from the approver.
- **Approved** -- revision approved by the approver.
- **Requested changes** -- changes to the revision requested by the approver.
- **Invalidated** -- an `Approved` request on a revision that had another approver mark `Requested changes` -- or a non-required `Requested changes` status on an approved revision. Applied once a new revision is made.
- **Outdated** -- set on any `Pending` requests if a new revision is made.
- **Rescinded** -- set on any request removed (deleted) by an assigner.

Note that the **Rescinded** state takes the place of **Deleted** for auditing purposes. Approval requests are only deleted if the `parent entity` (or its associated revision) is deleted.

## Permissions

- `Administer settings for Workbench Approver` allows access to module configuration. This permission is not yet implemented, since the module has no settings.
- `Assign users to approve content` allows users to assign other users to approve a content revision. Users with this permission can also edit approval requests to change their deadline and required status. Note that the edit action is not restricted to the creator of the approval request, nor is the assignment based on the creator of the content.
- `Let users be assigned to approve content` allows users to be assigned to approve a content revision.
- `View the editorial approval status of content` allows users to view approval requests but not to interact with them. Access to this permission can help explain why content transitions may be blocked.

These are considered `tiered` permissions. The optimal assignment is as follows:

- `Site administrators` -- all permissions.
- `Editor` -- the assign / use / and view permissions.
- `Content creator` -- the use and view permissions.
- `Authenticated users` -- the view permission (optional).

Note that permissions are **global** for content. Further development to allow permission per content type may be a desired feature.

## Approvals block

The module provides a block. It only appears on the latest revision of a node for users with the `view workbench approver` or `use workbench approver` permission.

The block provides the following information and options:

- For admins who can assign approvers.
  - The current status of approvals for the revision.
- For users who can approve content
  - The current status of approvals for the revision.
  - `Approve` or `Request changes` buttons that launch a submission form.
- For users who can view approvals
  - The current status of approvals for the revision.
  - Note that these users must have another permission that allows them to see the unpublished revision.

Place the block in your layout as normal. Additional configuration is optional, as path-level restrictions are enforced by the block itself.

Note that we do not auto-install this block to allow its placement to fit your site theme.

## Content moderation changes

Workbench Approver fundamentally alters how Content moderation works.

- When a revision is awaiting approval, transitions out of the current moderation state are forbidden. These `blocking` approvals exist if:
  - The approval request is `required`
  - And the approval request is either `Pending` or `Requested changes`
- If an approval is not required -- or if all required approvals are `Approved` -- content may transition to a new state.

In addition, the module changes who can access the latest revision of a piece of content. The following rules apply:

- Existing rules for revision access are respected.
- If a user is an assigned approver on the latest revision, they are granted access to the latest revision.

### Notes on workflow configuration

As noted above, if the latest revision is in a moderation state that does not support transitioning to itself, approvals may not be assigned.

For instance, the default workflow for content moderation allows `draft` to `draft` revisions, so content revisions in the `draft` state can have reviewers assigned. Content in an `archived` state cannot transition to `archived` (only to `draft` or `published`), so it cannot be assigned for approval.

The fundamental assumption is that approvals form a feedback loop within a moderation state. If you wish to allow approvals, each state subject to approval needs to allow transitions to itself.

## Views integration

The module creates a new `approver` entity type that provides a base table for Views.

The following database tables are exposed to Views:

- `workbench_approver_field_data` -- data for the current revision of an approval request.
- `workbench_approver_field_revision` -- data for the revisions of an approval request.

The module also ships with several views:

- `views.view.workbench_approver_admin_by_content.yml` -- content for the path `/admin/content/approvals`.
- `views.view.workbench_approver_admin_by_user.yml` -- content for the path `/admin/content/approvers`.
- `views.view.workbench_approver_default_tables.yml` -- content for the path `/node/{node_id}/approver`.
- `views.view.workbench_approver_revision_history.yml` -- optional content for the path /node/{node_id}/approver`.
- `views.view.workbench_approver_workbench_tab.yml` -- content for the Workbench tab at `/admin/workbench/content/approvals`.

### Joins and relationships

The above tables auto-join to the `user` tables. Additional joins are available via views relationships:

- `node` -- pull data from the `node` parent entity.
- `node_revision` -- pull data from `node revisions` on the parent entity.
- `content_moderation` -- pull revision state information for the entity revision that the approval request is for.

### Custom plugins

- `workbench_approver_status` -- a filter that provides a select list for [statuses](#approval-statuses).
- `workbench_approver_latest_revision` -- a field that provides a link to the latest revision of the parent entity. There is a core Drupal issue to provide this feature: [#2906455](https://www.drupal.org/project/drupal/issues/2906455)
- There is also outstanding work to put a date filter (with calendar selection) into Drupal core. Until that is done, we user the [date_filter module](https://drupal.org/project/date_filter).

## Workbench Access integration

By default, any user with the `Let users be assigned to approve content` permission can be assigned to approve any node. When the optional `workbench_approver_access` module is enabled, the list of available users is limited to users who are in the assigned Workbench Access section(s) that the node is in.

This module shows an example of the [EventSubscriber](#events) defined by the Workbench Approver module.

When this module is enabled, Workbench Access rules are overlaid on approval request access such that:

- Workbench Access rules determine who can view approval requests
- Workbench Access rules determine who can assign approval requests
- Workbench Access rules determine who can be assigned to approve revisions

### Use with Access Unpublished module

The [Access Unpublished](https://drupal.org/project/access_unpublished) module allows can allow anonymous users to view the latest revision tab with the proper permission. If using this module with Workbench Approver Access, these links will return access denied unless you allow the anonymous user the following:

* Grant the `Bypass Workbench Access permissions` permission from Workbench Access.

If you grant this permission, be sure that the anonymous role does not have permission to edit or delete content, as this may have unintended consequences.

## Technical architecture

The module is still under active development and testing.

### The `approver` entity type

The module creates an `approver` content entity which defines an `approval request` that must be attached to another `base entity`. The approval request tracks revisions and the language of the base entity.

#### Entity schema

- See `Approver::baseFieldDefinitions()` for the storage fields.

Note that the `assigner` is the entity owner (the `uid`), and the `approver` is stored as `approver_uid`.

### Routes

The module creates the following Drupal routes, accessible by users with the [proper permissions](#permissions):

- `/node/{node_id}/approver` -- the `Approve` tab for nodes.
- `/approver/{id}/{form_id}` -- form callbacks for approval assignment and resolution.
- `/admin/content/approvals` -- a sub-tab on the Content admin page, showing approval requests sorted by content.
- `/admin/content/approvers` -- a sub-tab on the Content admin page, showing approval requests sorted by user.
- `/admin/workbench/content/approvals` -- a tab for the optional [Workbench](https://drupal.org/project/workbench) module.

### Services

- `workbench_approver.manager` provides a bridge between approval requests and their parent entity. Used for finding the approval state of an entity. Most of its methods take a `ContentEntityInterface` as an argument.
- `workbench_approver.state_transition_validation` decorated (overrides) the core `content_moderation.state_transition_validation` service and restricts the allowed transitions based on approvals. See the notes under [Content moderation changes](#content-moderation-changes).
- `access_check.approve_latest_revision` decorates (overrides) the core `access_check.latest_revision` service which provides the `_content_moderation_latest_version` tag on routes. See the notes under [Content moderation changes](#content-moderation-changes).
- `access_check.view_approvals` provides a route tag `_workbench_approver_revisions` for handling access to the `Approve` tab on content.

### Events

The module provides an event for altering query results when finding users who can be assigned as approvers: `workbench_approver.get_potential_approvers`.

This event is fired as part of the autocomplete callback used by the `Add approval request` form. The call stack is worth noting here in case any developers need to change the list of available approvers:

- The `workbench_approver` EntityReferenceSelection handler `ApproverSelection` extends core `UserSelection` to filter users by the proper permission.
- `ApproverSelection::buildEntityQuery()` dispatches to `ApproverManager::getPotentialApproverIds()` to find users with the proper permissions who are not already assigned to the entity revision.
- `ApproverManager::getPotentialApproverIds()` compiles a list of user ids and dispatches a `GetPotentialApproversEvent`.
- `WorkbenchApproverAccessSubscriber` responds to the event and uses its `modifyResults()` method to apply additional filters to the list of users.

In the future, this event subscriber may move to the Workbench Access module.

## Troubleshooting and FAQ

### Views schema errors

There are core errors in views schema handling, so schema checking is disabled in tests.

* `display.default.display_options.fields.deadline.settings.time_diff.description`
  * https://www.drupal.org/project/drupal/issues/3400522

### Multilingual support

The approval request entity type stores language information and *should* be compatible with multilingual content. However, it remains to be tested fully.

## Credits

Icons created by iconmonstr and used [under license](https://iconmonstr.com/license/).
