<?php

namespace Drupal\workbench_approver_access\Access;

use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workbench_approver\Access\ApproveLatestRevisionCheck;
use Symfony\Component\Routing\Route;

/**
 * Grants access to the latest revision tab.
 */
class AccessApproveLatestRevisionCheck extends ApproveLatestRevisionCheck {

  /**
   * The originally decorated access check.
   *
   * @var \Drupal\Core\Routing\Access\AccessInterface
   */
  protected $accessCheck;

  /**
   * The parent decorator.
   *
   * @var \Drupal\workbench_approver\Access\ApproveLatestRevisionCheck
   */
  protected $parentCheck;

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * Constructs a new LatestRevisionCheck.
   *
   * @param \Drupal\Core\Routing\Access\AccessInterface $access_check
   *   Latest revision access check to decorate.
   * @param \Drupal\workbench_approver\Access\ApproveLatestRevisionCheck $check
   *   The parent decorator from workbench_approver.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_information
   *   The moderation information service.
   */
  public function __construct(AccessInterface $access_check, ApproveLatestRevisionCheck $check, ModerationInformationInterface $moderation_information) {
    $this->accessCheck = $access_check;
    $this->parentCheck = $check;
    $this->moderationInfo = $moderation_information;
  }

  /**
   * {@inheritdoc}
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account) {
    $access_result = AccessResult::neutral();
    $entity = $this->loadEntity($route, $route_match);
    // Only run this check on moderated entities.
    if ($this->moderationInfo->hasPendingRevision($entity) && $this->moderationInfo->isModeratedEntity($entity)) {
      // The entity hook is the direct method for getting access.
      // Update here is a sufficient check to learn if the entity
      // is within scope of the user.
      $access_result = workbench_access_entity_access($entity, 'update', $account);
      // A neutral result from the above grants access because workbench_access
      // only returns neutral or forbidden.
      if ($access_result->isNeutral()) {
        $access_result = $this->parentCheck->access($route, $route_match, $account);
      }
    }

    return $access_result;
  }

}
