<?php

namespace Drupal\workbench_approver\Access;

use Drupal\content_moderation\Access\LatestRevisionCheck;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\Routing\Route;

/**
 * Grants access to the Approve tab.
 */
class ViewApprovalsCheck extends LatestRevisionCheck {

  /**
   * The decorated access check.
   *
   * @var \Drupal\Core\Routing\Access\AccessInterface
   */
  protected $accessCheck;

  /**
   * The moderation information service.
   *
   * @var \Drupal\content_moderation\ModerationInformationInterface
   */
  protected $moderationInfo;

  /**
   * ApproveLatestRevisionCheck constructor.
   *
   * @param \Drupal\Core\Routing\Access\AccessInterface $access_check
   *   Latest revision access check to decorate.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderation_information
   *   The moderation information service.
   */
  public function __construct(AccessInterface $access_check, ModerationInformationInterface $moderation_information) {
    $this->accessCheck = $access_check;
    $this->moderationInfo = $moderation_information;
  }

  /**
   * {@inheritdoc}
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account) {
    // This tab should not show up unless there's a reason to show it.
    $entity = $this->loadEntity($route, $route_match);
    if ($this->moderationInfo->isModeratedEntity($entity)) {
      // Check the global permissions first.
      $access_result = AccessResult::allowedIfHasPermission($account, 'view workbench approver information');

      return $access_result->addCacheableDependency($entity);
    }

    return AccessResult::forbidden('No pending revision for moderated entity.')->addCacheableDependency($entity);
  }

}
