<?php

namespace Drupal\workbench_approver;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workbench_approver\Entity\ApproverInterface;

/**
 * The access control handler for the `approver` entity type.
 *
 * @see \Drupal\workbench_approver\Entity\ContentModerationState
 */
class ApproverAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    // View access is separate.
    if ($operation === 'view') {
      if ($account->hasPermission('view workbench approver information')) {
        return AccessResult::allowed()->cachePerPermissions();
      }
      else {
        return AccessResult::forbidden("The 'view workbench approver information' permission is required.")->cachePerPermissions();
      }
    }

    assert($entity instanceof ApproverInterface);
    // Special case to update your assigned approval.
    // AccountInterface::id() is returning a string here?!?
    if ($operation === 'update' && $entity->getApproverId() === intval($account->id())) {
      if ($account->hasPermission('use workbench approver')) {
        return AccessResult::allowed()->cachePerPermissions();
      }
      else {
        return AccessResult::forbidden("The 'use workbench approver' permission is required.")->cachePerPermissions();
      }
    }

    // All other cases.
    if ($account->hasPermission('assign workbench approver')) {
      return AccessResult::allowed()->cachePerPermissions();
    }
    else {
      return AccessResult::forbidden("The 'assign workbench approver' permission is required.")->cachePerPermissions();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    $account = $this->prepareUser($account);

    if ($account->hasPermission('assign workbench approver')) {
      return AccessResult::allowed()->cachePerPermissions();
    }
    else {
      return AccessResult::forbidden("The 'assign workbench approver' permission is required.")->cachePerPermissions();
    }
  }

}
