<?php

namespace Drupal\workbench_approver;

use Drupal\views\EntityViewsData;

/**
 * Provides the views data for the approver entity type.
 */
class ApproverViewsData extends EntityViewsData {

  /**
   * {@inheritdoc}
   */
  public function getViewsData() {
    $data = parent::getViewsData();

    // @todo In future, we may support more types.
    $entity_type_info = [
      'node' => [
        'table' => 'node_field_data',
        'entity_type' => 'node',
        'key' => 'nid',
        'title' => $this->t('Content'),
        'workbench_approver_key' => 'content_entity_id',
      ],
      'node_revision' => [
        'table' => 'node_field_revision',
        'key' => 'vid',
        'entity_type' => 'node',
        'title' => $this->t('Content revision'),
        'workbench_approver_key' => 'content_entity_revision_id',
      ],
      'content_moderation' => [
        'table' => 'content_moderation_state_field_data',
        'key' => 'content_entity_id',
        'entity_type' => 'node',
        'title' => $this->t('Content moderation state'),
        'workbench_approver_key' => 'content_entity_id',
      ],
    ];
    $views_base_table = 'workbench_approver_field_data';

    foreach ($entity_type_info as $type => $info) {
      // We add relationships, not direct JOINS, since we are a special case.
      $data[$views_base_table][$info['key']]['relationship'] = [
        'title' => $info['title'],
        'help' => $this->t('The %title this approval is for', ['%title' => $info['title']]),
        'base' => $info['table'],
        'base field' => $info['key'],
        'relationship field' => $info['workbench_approver_key'],
        'id' => 'standard',
        'label' => $info['title'],
        'extra' => [
          [
            'field' => 'content_entity_type_id',
            'value' => $info['entity_type'],
            'table' => $views_base_table,
          ],
          ['left_field' => 'langcode', 'field' => 'langcode'],
        ],
      ];

      // @todo It looks like we have to reverse this relationship for {node}.
      // Special handling for content_moderation.
      if ($type === 'content_moderation') {
        $data[$views_base_table][$info['key']]['relationship']['extra'] = [
          ['left_field' => 'langcode', 'field' => 'langcode'],
          ['left_field' => 'workflow_id', 'field' => 'workflow'],
          ['left_field' => 'content_entity_id', 'field' => 'content_entity_id'],
        ];
      }
    }

    // The approver_uid field is a user field.
    $data['workbench_approver_field_data']['approver_uid']['filter'] = ['id' => 'user_name'];
    $data['workbench_approver_field_revision']['approver_uid']['filter'] = ['id' => 'user_name'];

    // Custom status filter.
    $data['workbench_approver_field_data']['status']['filter'] = ['id' => 'workbench_approver_status'];
    $data['workbench_approver_field_revision']['status']['filter'] = ['id' => 'workbench_approver_status'];

    // Custom revision link field.
    // See https://www.drupal.org/project/drupal/issues/2906455.
    $data['node_revision']['workbench_approver_latest_revision'] = [
      'title' => $this->t('Link to latest revision'),
      'field' => [
        'id' => 'workbench_approver_latest_revision',
        'help' => $this->t('Provides a link to the latest revision of an entity. If the latest revision is the default revision, provide a link to that instead.'),
      ],
    ];

    return $data;
  }

}
