<?php

namespace Drupal\workbench_approver\Event;

use Drupal\Component\EventDispatcher\Event;

/**
 * Event for altering query results when finding users.
 */
class GetPotentialApproversEvent extends Event {

  /**
   * The array of users ids returned by a query.
   *
   * @var array
   */
  protected $potentialApprovers;

  /**
   * The arguments passed by to the query.
   *
   * @var array
   */
  protected $arguments;

  /**
   * GetPotentialApproversEvent constructor.
   *
   * @param array $potential_approvers
   *   The list of potential content approvers returned by
   *   ApproverManager::getPotentialApproverIds().
   * @param array $arguments
   *   The arguments passed by to the query.
   */
  public function __construct(array $potential_approvers, array $arguments) {
    $this->potentialApprovers = $potential_approvers;
    $this->arguments = $arguments;
  }

  /**
   * Gets the arguments passed to the query.
   *
   * @return array
   *   An array of query arguments.
   */
  public function getArguments():array {
    return $this->arguments;
  }

  /**
   * Gets the list of approvers.
   *
   * @return array
   *   An array of user ids.
   */
  public function getPotentialApproverIds(): array {
    return $this->potentialApprovers;
  }

  /**
   * Sets a new value for the potential approvers.
   *
   * @param array $approver_uids
   *   An array of user uids.
   */
  public function setPotentialApprovers(array $approver_uids): void {
    $this->potentialApprovers = $approver_uids;
  }

}
