<?php

namespace Drupal\workbench_approver\Form;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form for a reviewer view current notes.
 *
 * This isn't really a form, but Drupal makes it easy
 * to hook this into our operations pattern.
 */
class NotesForm extends ApproverFormBase {

  /**
   * {@inheritdoc}
   */
  public $allowedFields = [];

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $formState) {
    $form = parent::form($form, $formState);
    $this->removeFields($form);

    $notes = $this->entity->get('notes')->value;

    if ($notes !== NULL) {
      $notes = Xss::filter($notes);
    }
    else {
      $notes = $this->t('There are no current notes to display.');
    }

    $form['view_notes'] = [
      '#markup' => $notes,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getSubmitTitle() {
    return $this->t('Approve revision');
  }

  /**
   * {@inheritdoc}
   */
  public function actions(array $form, FormStateInterface $form_state) {
    $actions = [];
    $this->addCancelAction($actions);

    return $actions;
  }

}
