<?php

namespace Drupal\workbench_approver\Plugin\EntityReferenceSelection;

use Drupal\user\Plugin\EntityReferenceSelection\UserSelection;

/**
 * Provides specific access control for the user entity type.
 *
 * If we need more specificity, use a Deriver to create more.
 *
 * @todo Validation.
 *
 * @EntityReferenceSelection(
 *   id = "workbench_approver",
 *   label = @Translation("Filtered user selection"),
 *   entity_types = {"user"},
 *   group = "workbench_approver",
 *   weight = 1,
 * )
 */
class ApproverSelection extends UserSelection {

  /**
   * {@inheritdoc}
   */
  protected function buildEntityQuery($match = NULL, $match_operator = 'CONTAINS') {
    $query = parent::buildEntityQuery($match, $match_operator);
    if (isset($this->configuration)) {
      $handler_settings = $this->configuration;
    }

    // @phpstan-ignore-next-line
    $approver_manager = \Drupal::service('workbench_approver.manager');
    $allowed_uids = $approver_manager->getPotentialApproverIds($handler_settings['filters']);

    // If no editors, return no results.
    if (count($allowed_uids) > 0) {
      $query->condition('uid', $allowed_uids, 'IN');
    }
    else {
      $query->condition('uid', -1);
    }

    return $query;
  }

}
