<?php

namespace Drupal\workbench_approver\Plugin\views\field;

use Drupal\Core\Entity\RevisionableInterface;
use Drupal\views\Entity\Render\EntityTranslationRenderTrait;
use Drupal\views\Plugin\views\field\EntityLink;
use Drupal\views\ResultRow;

/**
 * Field handler to view the latest revision of an entity.
 *
 * This should be replaced by core when the patch lands.
 *
 * See https://www.drupal.org/project/drupal/issues/2906455.
 *
 * If you want to link to the entity title and not to
 * 'View latest revision', use the Views option to
 * rewrite link text.
 *
 * @ViewsField("workbench_approver_latest_revision")
 */
class LinkToLatestRevision extends EntityLink {

  use EntityTranslationRenderTrait;

  /**
   * {@inheritdoc}
   */
  protected function getUrlInfo(ResultRow $row) {
    $entity = $this->getEntity($row);
    assert($entity instanceof RevisionableInterface);
    $template = !$entity->isLatestRevision() && $entity->hasLinkTemplate('latest-version') ? 'latest-version' : $this->getEntityLinkTemplate();

    if ($this->languageManager->isMultilingual()) {
      $entity = $this->getEntityTranslationByRelationship($entity, $row);
      $this->addLangcode($row);
    }

    return $entity->toUrl($template);
  }

  /**
   * {@inheritdoc}
   */
  protected function getDefaultLabel() {
    return $this->t('View latest revision');
  }

}
