<?php

namespace Drupal\workbench_approver_validation_test;

use Drupal\content_moderation\StateTransitionValidationInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflows\StateInterface;
use Drupal\workflows\WorkflowInterface;

/**
 * Defines a class for decorating state transition validation.
 */
class StateTransitionValidation implements StateTransitionValidationInterface {

  /**
   * Original service.
   *
   * @var \Drupal\content_moderation\StateTransitionValidationInterface
   */
  protected $original;

  /**
   * Constructs a new StateTransitionValidation.
   *
   * @param \Drupal\content_moderation\StateTransitionValidationInterface $original
   *   Decorated state validation service.
   */
  public function __construct(StateTransitionValidationInterface $original) {
    $this->original = $original;
  }

  /**
   * {@inheritdoc}
   */
  public function getValidTransitions(ContentEntityInterface $entity, AccountInterface $user) {
    $transitions = $this->original->getValidTransitions($entity, $user);
    // Deny the archived transition.
    foreach ($transitions as $id => $transition) {
      if ($transition->to()->id() !== 'archived') {
        $allowed_transitions[$id] = $transition;
      }
    }

    return $allowed_transitions;
  }

  /**
   * {@inheritdoc}
   */
  public function isTransitionValid(WorkflowInterface $workflow, StateInterface $original_state, StateInterface $new_state, AccountInterface $user, ContentEntityInterface $entity) {
    return $this->original->isTransitionValid($workflow, $original_state, $new_state, $user, $entity);
  }

}
