<?php

namespace Drupal\Tests\workbench_approver\Functional;

/**
 * Tests for the add approver form.
 *
 * @group workbench_approver
 */
class AddApproverFormTest extends WorkbenchApproverTestBase {

  /**
   * Tests the add approver form.
   *
   * @covers \Drupal\workbench_approver\Form\AddApproverForm
   */
  public function testAddApproverForm() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);
    $approved_node = $this->createRevision($approved_node);

    $path = '/node/' . $approved_node->id() . '/approver';

    $this->drupalLogin($this->adminUser);
    $this->drupalGet($path);

    $this->assertSession()->buttonExists('Add approval')->click();

    $this->submitForm([
      'approver_uid' => $this->editorUser->label() . ' (' . $this->editorUser->id() . ')',
      'required' => TRUE,
      'notes' => 'Test notes',
    ], 'Request approval');

    $this->drupalGet($path);

    // Check that the user was added.
    $this->assertSession()->pageTextContains($this->editorUser->label());
    $this->assertSession()->pageTextContains('Yes');

    // Now remove the user.
    $approval = $this->approverStorage->load(1);
    $this->assertTrue(intval($approval->id()) === 1);
    $delete_path = '/approver/' . $approval->id() . '/delete';
    $this->drupalGet($delete_path);
    $this->assertSession()->buttonExists('Delete request')->click();

    // Check that the user was hidden.
    $this->drupalGet($path);
    $this->assertSession()->pageTextNotContains($this->editorUser->label());
    $this->assertSession()->pageTextNotContains('Yes');

    // Now re-add without being required.
    $this->assertSession()->buttonExists('Add approval')->click();

    $this->submitForm([
      'approver_uid' => $this->editorUser->label() . ' (' . $this->editorUser->id() . ')',
      'required' => FALSE,
      'notes' => 'Test notes',
    ], 'Request approval');

    // Check that the user was added but not required.
    $this->assertSession()->pageTextContains($this->editorUser->label());
    $this->assertSession()->pageTextNotContains('Yes');
  }

}
