<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\node\NodeInterface;
use Drupal\workbench_approver\Entity\ApproverInterface;

/**
 * Tests for lists of available approvers.
 *
 * @todo This may be better as a Kernel test.
 *
 * @group workbench_approver
 */
class AllowedApproversTest extends WorkbenchApproverTestBase {

  /**
   * Tests that our roles list is as expected.
   *
   * @covers \Drupal\workbench_approver\Services\ApproverManager::getAllowedRoles()
   */
  public function testGetAllowedRoles() {
    $expected_rids = [
      'admin',
      'editor',
      'workflow_admin',
    ];

    $not_expected_rids = [
      'observer',
    ];

    $allowed_rids = $this->approverManager->getAllowedRoles();
    $this->assertEquals($expected_rids, $allowed_rids, 'Expected roles match');
    $this->assertEmpty(array_intersect($not_expected_rids, $allowed_rids));
  }

  /**
   * Tests that our user list is as expected.
   *
   * @covers \Drupal\workbench_approver\Services\ApproverManager::getPotentialApproverIds()
   */
  public function testGetPotentialApprovers() {
    $expected_uids = [
      $this->adminUser->id(),
      $this->editorUser->id(),
      $this->anotherEditorUser->id(),
      $this->workflowAdminUser->id(),
    ];

    $not_expected_uids = [
      $this->observerUser->id(),
    ];

    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'published',
    ];
    $approved_node = $this->drupalCreateNode($values);
    $approved_node = $this->createRevision($approved_node);
    assert($approved_node instanceof NodeInterface);

    $arguments = [
      'content_entity_id' => $approved_node->id(),
      'content_entity_revision_id' => $approved_node->getRevisionId(),
      'content_entity_type_id' => 'node',
      'workflow_id' => $this->workflow->id(),
    ];

    $this->checkApprovers($arguments, $expected_uids, $not_expected_uids);

    // Now add the admin user and check again.
    $approver = $this->addApprovalRequest($this->adminUser, $approved_node);

    $expected_uids = [
      $this->editorUser->id(),
      $this->anotherEditorUser->id(),
      $this->workflowAdminUser->id(),
    ];

    $not_expected_uids = [
      $this->adminUser->id(),
      $this->observerUser->id(),
    ];

    $this->checkApprovers($arguments, $expected_uids, $not_expected_uids);

    // Given that the user is set to APPROVAL_RESCINDED, they can be re-added.
    $approver->set('status', ApproverInterface::APPROVAL_RESCINDED);
    $approver->save();

    $expected_uids = [
      $this->adminUser->id(),
      $this->editorUser->id(),
      $this->anotherEditorUser->id(),
      $this->workflowAdminUser->id(),
    ];

    $not_expected_uids = [
      $this->observerUser->id(),
    ];

    $this->checkApprovers($arguments, $expected_uids, $not_expected_uids);
  }

}
