<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\Core\Entity\EntityInterface;

/**
 * Tests for content on the admin by content list.
 *
 * @group workbench_approver
 */
class ApproversByUserTest extends WorkbenchApproverTestBase {

  /**
   * Test access to the tab.
   */
  public function testAccessToPage() {
    $path = '/admin/content/approvers';

    $users = [
      NULL,
      $this->adminUser,
      $this->editorUser,
      $this->observerUser,
      $this->workflowAdminUser,
    ];

    foreach ($users as $user) {
      if ($user instanceof EntityInterface) {
        $this->drupalLogin($user);
        $this->drupalGet($path);
        $this->assertSession()->statusCodeEquals(200);
      }
      else {
        $this->drupalGet($path);
        $this->assertSession()->statusCodeEquals(403);
      }
    }
  }

  /**
   * Tests content on the tab.
   */
  public function testContentAdmin() {
    $path = '/admin/content/approvers';

    $nodes = [];
    for ($i = 1; $i <= 5; $i++) {
      $title = "Test node $i";
      $users = [$this->editorUser];
      $options = ['required' => FALSE];
      $nodes[$i] = $this->createNodeWithApprovalRequests($title, $users, $options);
    }

    $this->drupalLogin($this->editorUser);
    $this->drupalGet($path);
    $this->assertSession()->pageTextContains('Content approvals by user');

    // Fields check.
    $this->assertSession()->fieldExists('edit-approver-uid');
    $this->assertSession()->fieldExists('edit-uid');
    $this->assertSession()->selectExists('edit-required');
    $this->assertSession()->fieldExists('edit-deadline-date');
    $this->assertSession()->fieldExists('edit-deadline-1-date');
    $this->assertSession()->buttonExists('Apply');

    $node_labels = $this->xpath('//td[contains(@class, "views-field-title")]');
    $approver_labels = $this->xpath('//td[contains(@class, "views-field-approver-uid")]');
    $required_values = $this->xpath('//td[contains(@class, "views-field-required")]');
    $statuses = $this->xpath('//td[contains(@class, "views-field-status")]');
    $creation_dates = $this->xpath('//td[contains(@class, "views-field-created")]');
    $deadlines = $this->xpath('//td[contains(@class, "views-field-deadline")]');
    $assigners = $this->xpath('//td[contains(@class, "views-field-uid")]');

    $delta = 0;
    foreach ($nodes as $node) {
      $approver = $this->approverManager->getApprovalRequestByUser($node, $this->editorUser);

      $this->assertEquals(trim($node_labels[$delta]->getText()), $node->label());

      $this->assertEquals(trim($approver_labels[$delta]->getText()), $approver->getApproverLabel());

      $required = (bool) $approver->get('required')->value ? 'Yes' : 'No';
      $this->assertEquals(trim($required_values[$delta]->getText()), $required);

      $created = date('d-M-Y', $approver->get('created')->value);
      $this->assertEquals(trim($creation_dates[$delta]->getText()), $created);

      $deadline = date('d-M-Y', $approver->get('deadline')->value);
      $this->assertEquals(trim($deadlines[$delta]->getText()), $deadline);

      $status = $approver->getStatusName();
      $this->assertEquals(trim($statuses[$delta]->getText()), $status);

      $this->assertEquals(trim($assigners[$delta]->getText()), $approver->getAssignerLabel());

      $delta++;
    }
    $headers = [
      'Title',
      'Approver',
      'Required',
      'Status',
      'Due date',
      'Requested',
      'Requested by',
      'Operations',
    ];
    foreach ($headers as $header) {
      $this->assertSession()->pageTextContains($header);
    }
  }

}
