<?php

namespace Drupal\Tests\workbench_approver\Functional;

/**
 * Tests approval handling when content is deleted.
 *
 * @group workbench_approver
 */
class ContentEntityDeleteTest extends WorkbenchApproverTestBase {

  /**
   * Tests when a node is deleted.
   *
   * @covers \Drupal\workbench_approver\Services\ApproverManagerInterface::entityDelete()
   *
   * All related approvals should be deleted.
   */
  public function testContentEntityDelete() {
    $approved_node = $this->createNodeWithApprovalRequests();

    // Delete the node.
    $this->drupalLogin($this->adminUser);
    $delete_path = '/node/' . $approved_node->id() . '/delete';
    $this->drupalGet($delete_path);

    $return_path = '/user/' . $this->adminUser->id();
    $this->assertSession()->buttonExists('Delete')->click();
    $this->assertSession()->addressEquals($return_path);

    $this->assertApprovers($approved_node, 0);
  }

  /**
   * Tests entity deletion when multiple revisions have approvers.
   *
   * @covers \Drupal\workbench_approver\Services\ApproverManagerInterface::entityDelete()
   *
   * All related approvals should be deleted.
   */
  public function testContentEntityDeleteMultiple() {
    // Test with multiple assigned revisions.
    $new_node = $this->createNodeWithApprovalRequests();
    $new_revision = $this->createRevision($new_node);
    $this->addApprovalRequest($this->adminUser, $new_revision);
    $this->addApprovalRequest($this->editorUser, $new_revision);

    // Assert creation.
    $this->assertApprovers($new_revision, 2);

    // Delete the node.
    $this->drupalLogin($this->adminUser);
    $delete_path = '/node/' . $new_revision->id() . '/delete';
    $this->drupalGet($delete_path);
    $this->assertSession()->buttonExists('Delete')->click();

    $parameters = [
      'content_entity_type_id' => $new_revision->getEntityTypeId(),
      'content_entity_id' => $new_revision->id(),
      'langcode' => $new_revision->language()->getId(),
    ];

    // Note that ApprovalManager::getApprovalRequests() filters "RESCINDED.".
    $approvers = $this->approverStorage->loadByProperties($parameters);
    $this->assertEquals(0, count($approvers));
  }

}
