<?php

namespace Drupal\Tests\workbench_approver\Functional;

/**
 * Tests that the Content Moderation form appears as expected.
 *
 * This is an explicit test for when another module decorates
 * the validation service.
 *
 * @group workbench_approver
 */
class ContentModerationValidationInnerTest extends ContentModerationIntegrationTest {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'content_moderation',
    'workbench_approver',
    'workbench_approver_validation_test',
  ];

  /**
   * Overrides the base test to ensure that our service is called.
   */
  public function testEditForm() {
    // Create a new node and do nothing with it.
    $values = [
      'type' => 'approved',
      'title' => 'Non-assigned node',
      'moderation_state' => 'draft',
    ];
    $node = $this->drupalCreateNode($values);
    $path = '/node/' . $node->id() . '/edit';

    // We expect to find the states allowed by the test module.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet($path);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Current state Draft');
    $this->assertSession()->optionExists('moderation_state[0][state]', 'draft');
    $this->assertSession()->optionExists('moderation_state[0][state]', 'published');
    // Out test module explicitly rejects the archived state.
    $this->assertSession()->optionNotExists('moderation_state[0][state]', 'archived');

    $this->createRevision($node, 'published');
    $this->drupalGet($path);
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Current state Published');
    $this->assertSession()->optionExists('moderation_state[0][state]', 'draft');
    $this->assertSession()->optionExists('moderation_state[0][state]', 'published');
    // Out test module explicitly rejects the archived state.
    $this->assertSession()->optionNotExists('moderation_state[0][state]', 'archived');
  }

}
