<?php

namespace Drupal\Tests\workbench_approver\Functional;

/**
 * Tests for access to approval request creation / viewing.
 *
 * @group workbench_approver
 */
class FormAccessTest extends WorkbenchApproverTestBase {

  /**
   * Tests for access to approval forms.
   *
   * Note that `add approver` is embedded in the
   * node tab and not otherwise accessible. Tests for that
   * are covered in AccessTest.
   *
   * Other form paths are "guessable" if the user knows
   * the approval request id, and should have access checks.
   */
  public function testApprovalFormAccess() {
    $values = [
      'type' => 'page',
      'title' => 'Page node',
      'moderation_state' => 'published',
    ];
    $node1 = $this->drupalCreateNode($values);
    $revision1 = $this->createRevision($node1);

    // Testing paths.
    $base_path = '/node/' . $node1->id();
    $this->drupalGet($base_path);

    // Create an approval request.
    $approval = $this->addApprovalRequest($this->editorUser, $revision1);

    $users = [
      NULL,
      $this->editorUser,
      $this->adminUser,
      $this->observerUser,
    ];

    // Rules:
    // - Only the assigner/owner can edit or delete.
    // - The assigner/owner and the approver can approve/reject.
    // - Any role with the view permission can see notes.
    $paths = [
      '/approver/' . $approval->id() . '/edit' => [
        $this->adminUser->id(),
      ],
      '/approver/' . $approval->id() . '/approve' => [
        $this->editorUser->id(),
        $this->adminUser->id(),
      ],
      '/approver/' . $approval->id() . '/reject' => [
        $this->editorUser->id(),
        $this->adminUser->id(),
      ],
      '/approver/' . $approval->id() . '/delete' => [
        $this->adminUser->id(),
      ],
      '/approver/' . $approval->id() . '/notes'  => [
        $this->editorUser->id(),
        $this->adminUser->id(),
        $this->observerUser->id(),
      ],
    ];

    $this->assertApprovalPathAccess($paths, $users);
  }

}
