<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\node\NodeInterface;

/**
 * Tests for a message on the Revert revision form.
 *
 * @group workbench_approver
 */
class RevertFormAlterTest extends WorkbenchApproverTestBase {

  /**
   * Test case.
   */
  public function testRevertFormMessage() {
    $values = [
      'type' => 'approved',
      'title' => 'Approved node',
      'moderation_state' => 'draft',
    ];
    $draft_node = $this->drupalCreateNode($values);
    $this->assertEquals(1, $draft_node->getRevisionId());

    // Create a published revision.
    $published_node = $this->createRevision($draft_node, 'published');
    assert($published_node instanceof NodeInterface);
    $this->assertEquals(2, $published_node->getRevisionId());

    // Create a new revision and assign approvers.
    $revised_node = $this->createRevision($published_node);
    assert($revised_node instanceof NodeInterface);
    $this->assertEquals(3, $revised_node->getRevisionId());
    $this->addApprovalRequest($this->adminUser, $revised_node);
    $this->addApprovalRequest($this->editorUser, $revised_node);

    // Assert creation of approvals on the new draft.
    $this->assertApprovers($revised_node, 2);

    // Check that the original draft does not have approvals.
    $this->assertApprovers($draft_node, 0);

    // Login and check the revert form.
    $this->drupalLogin($this->adminUser);

    // Show approvals for debugging.
    $this->loadTestPaths($draft_node);

    $revert_path = '/node/' . $draft_node->id() . '/revisions/' . $draft_node->getRevisionId() . '/revert';
    $this->drupalGet($revert_path);
    $this->assertSession()->buttonExists('Revert');
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->pageTextContains('There are 2 outstanding approval requests for the latest revision.');
    $this->assertSession()->pageTextContains('If you revert to this revision, you will need to recreate any desired approval requests.');

    $this->assertSession()->buttonExists('Revert')->click();

    // When we revert, no new requests are available.
    $this->drupalGet($revert_path);
    $this->assertSession()->buttonExists('Revert');
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->pageTextNotContains('There are 2 outstanding approval requests for the latest revision.');
    $this->assertSession()->pageTextNotContains('If you revert to this revision, you will need to recreate any desired approval requests.');

    // Get the revision created by the revert action.
    $reverted_node = $this->getLatestNodeRevision($draft_node);
    $this->addApprovalRequest($this->adminUser, $reverted_node);

    // Assert creation of approvals on the new draft.
    $this->assertApprovers($reverted_node, 1);

    $this->drupalGet($revert_path);
    $this->assertSession()->buttonExists('Revert');
    $this->assertSession()->linkExists('Cancel');
    $this->assertSession()->pageTextContains('There is 1 outstanding approval request for the latest revision.');
    $this->assertSession()->pageTextContains('If you revert to this revision, you will need to recreate any desired approval requests.');
  }

}
