<?php

namespace Drupal\Tests\workbench_approver\Functional;

use Drupal\workbench_approver\Entity\ApproverInterface;

/**
 * Tests for content on the admin by content list.
 *
 * @group workbench_approver
 */
class UserBlockedTest extends WorkbenchApproverTestBase {

  /**
   * Tests user hooks (for blocked users).
   *
   * @covers ::workbench_approver_user_update()
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function testApprovalsWithBlockedApprover() {
    // There are two methods to block a user.
    $block_cancel_methods = [
      'user_cancel_block',
      'user_cancel_block_unpublish',
    ];

    foreach ($block_cancel_methods as $block_cancel_method) {
      // Select blocking method.
      $this->config('user.settings')->set('cancel_method', $block_cancel_method)->save();

      // Create user to block in different tests.
      $approverToBlock = $this->createUserWithAssignedRole('editor');

      $approved_node_1 = $this->drupalCreateNode([
        'type' => 'approved',
        'title' => 'Approved node 1',
        'moderation_state' => 'draft',
        'uid' => $this->editorUser,
      ]);

      $approved_node_2 = $this->drupalCreateNode([
        'type' => 'approved',
        'title' => 'Approved node 2',
        'moderation_state' => 'draft',
        'uid' => $this->editorUser,
      ]);

      // Create revisions and approvals.
      $this->drupalLogin($this->adminUser);

      /** @var \Drupal\Core\Entity\ContentEntityInterface $node_revision_1 */
      $node_revision_1 = $this->createRevision($approved_node_1);
      /** @var \Drupal\Core\Entity\ContentEntityInterface $node_revision_2 */
      $node_revision_2 = $this->createRevision($approved_node_2);

      $this->addApprovalRequest($approverToBlock, $node_revision_1, ['status' => ApproverInterface::APPROVAL_PENDING]);
      $this->addApprovalRequest($approverToBlock, $node_revision_2, ['status' => ApproverInterface::APPROVAL_PENDING]);

      // Block approver.
      $this->drupalGet('user/' . $approverToBlock->id() . '/cancel');
      $this->assertSession()->pageTextContains('Cancellation method');
      // Confirm user is blocked.
      $this->submitForm([], 'Confirm');
      // Confirm that the confirmation message made it through to the end user.
      $this->assertSession()->pageTextContains("Account {$approverToBlock->getAccountName()} has been disabled.");

      // Confirm changes in approvals.
      $approval_ids = $this->approverManager->getApprovalRequestIdsByEntityKeys('approver_uid', $approverToBlock->id());
      $this->assertNotEmpty($approval_ids);
      $approvals = $this->approverStorage->loadMultiple($approval_ids);
      $this->assertNotEmpty($approvals);
      /** @var \Drupal\workbench_approver\Entity\Approver $approval */
      foreach ($approvals as $approval) {
        $this->assertSame(ApproverInterface::APPROVAL_RESCINDED, $approval->getStatus(), 'Status of the Approval not changed as expected for blocked user.');
        $this->assertSame("The status of this request has been changed to \"{$approval->getStatusName()}\". The approver of this request with username \"{$approverToBlock->getAccountName()}\" has been blocked.",
          strip_tags($approval->get('notes')->getString()), 'The notes message in the approval is not the expected one.');
      }
    }
  }

}
